package temperature

import (
	"context"
	"fmt"
	"log"
	"os"

	"github.com/go-flucky/flucky/pkg/storage"
	"github.com/go-flucky/flucky/pkg/types"

	"github.com/go-flucky/flucky/pkg/cli"
	"github.com/go-flucky/flucky/pkg/config"
	"github.com/go-flucky/flucky/pkg/sensor"
	"github.com/spf13/cobra"
)

var readTemperatureCmd = &cobra.Command{
	Use:     "read",
	Short:   "Reading temperature values from different or specified sensors by arguments",
	Example: fmt.Sprintf("flucky temperature read\nflucky temperature read outdoor"),
	Run: func(cmd *cobra.Command, args []string) {

		// read configuration
		cnf, err := config.Read(*configFile)
		if err != nil {
			log.Fatalln(err)
		}

		// fetch all temperature sensors or sensors by args
		sensors := make([]sensor.Sensor, 0)
		if len(args) == 0 {
			sensors = cnf.GetTemperatureSensors(config.ENABLED)
		} else {
			sensors = cnf.GetTemperatureSensorsByName(args)
		}

		if len(sensors) == 0 {
			log.Fatalln("No sensors found, specified or configured")
		}

		ctx := context.Background()
		measuredValues, err := sensor.Read(ctx, sensors)
		if err != nil {
			log.Fatalln(err)
		}

		storage.Round(measuredValues, round)

		measuredValues = types.SelectMeasuredValues(types.MeasuredValueTypeTemperature, measuredValues)

		// print temperatures on stdout
		cli.PrintMeasuredValues(measuredValues, cnf, os.Stdout)

		// Save the new measured values, if desired
		if logs {

			if compression {
				measuredValues = storage.Compression(measuredValues)
			}

			storageEndpoint, err := cnf.GetStorageEndpointURL()
			if err != nil {
				log.Fatalln(err)
			}

			err = storage.Write(ctx, measuredValues, storageEndpoint)
			if err != nil {
				log.Fatalln(err)
			}
		}
	},
}

func init() {
	temperatureCmd.AddCommand(readTemperatureCmd)
	readTemperatureCmd.Flags().BoolVar(&logs, "logs", true, "Log temperature")
	readTemperatureCmd.Flags().BoolVar(&compression, "compression", true, "Compress measured with logged temperatures")
	readTemperatureCmd.Flags().Float64VarP(&round, "round", "r", 0.25, "Round values. The value 0 deactivates the function")
}
